icons = "__"..name.."__/graphics/icons/"

function geticon(icon, mod)
	local iconsize = 64
	if icon:match("x") then
		local split = icon:gmatch("([^x]+)")
		icon = split()
		iconsize = split()
	end
	if not icon:match("/graphics/") then
		if not mod then mod = name end
		icon = "__"..mod.."__/graphics/icons/"..icon..".png"
	end
	return icon, iconsize
end

function newproto(type, old, new, override, mult, graphics, gf)
	if data.raw[type][old] then 
		local proto = table.deepcopy(data.raw[type][old])
		proto.name = new
		proto.icon = icons..new..".png"
		if proto.minable and proto.minable.result then proto.minable.result = new end
		if proto.place_result then proto.place_result = new end
		if proto.take_result then proto.take_result = new end
		if proto.result then proto.result = new end
		if override then
			for key, val in pairs(override) do
				proto[key] = val
			end
		end
		if mult then
			for key, val in pairs(mult) do
				proto[key] = proto[key] * val
			end
		end
		if graphics then
			if not gf then gf = function(g) return g:gsub("__base__", "__"..name.."__") end end
			for _, val in ipairs(graphics) do
				proto[val].filename = gf(proto[val].filename)
			end
		end
		data:extend {proto}
		return(proto)
	end
	error("unknown prototype " .. old, 2)
end

function recipe(product, time, input, amount, enable)
	local ingreds = {}
	if type(input[#input]) == "string" then table.insert(input, 1) end
	local subtract = 0
	for i=1,#input,2 do
		i = i - subtract
		if type(input[i+1]) == "string" then
			table.insert(ingreds, {input[i], 1})
			subtract = subtract + 1
		else
			table.insert(ingreds, {input[i], input[i+1]})
		end
	end
	data:extend {{
		type = "recipe",
		name = product,
		enabled = enable or false,
		energy_required = time,
		ingredients = ingreds,
		result = product,
		result_count = amount or 1
	}}
end

function getpacks(packs, s)
	local ingreds = {}
	s = s or 1
	for i=s,math.min(s+2,#packs) do
		log(i-s+1)
		log(packs[i])
		if i-s+1 == 1 then
			table.insert(ingreds, {"automation-science-pack", packs[i]})
		elseif i-s+1 == 2 then
			table.insert(ingreds, {"logistic-science-pack", packs[i]})
		elseif i-s+1 == 3 then
			table.insert(ingreds, {"chemical-science-pack", packs[i]})
		end
	end
	if #packs > s+2 then
		for i=s+3,#packs,2 do
			table.insert(ingreds, {packs[i].."-science-pack", packs[i+1]})
		end
	end
	return ingreds
end

function geteffect(e)
	if type(e)=="string" then
		return {{type = "unlock-recipe", recipe = e}}
	end
	eff = {}
	for _, v in ipairs(e) do
		if type(v)=="string" then
			table.insert(eff, {type = "unlock-recipe", recipe = v})
		else
			table.insert(eff, v)
		end
	end
	return eff
end

function tech(name, icon, packs, effect, pre, order)
	-- parse sciencepacks
	local time, s = 30, 2
	if  packs[2]>4 then
		time = packs[2]
		s = 3
	end
	ingreds = getpacks(packs, s)
	-- parse icon
	local icon, iconsize = geticon(icon)
	-- add technology
	data:extend{{
			type = "technology",
			name = name,
			icon = icon,
			icon_size = iconsize,
			unit = {
				count = packs[1],
				time = time,
				ingredients = ingreds
			},
			effects = geteffect(effect),
			prerequisites = pre or {},
			order = order or ""
		}}
	return data.raw["technology"][name]
end

function inftech(name, icon, max, packs, effects, pre, order)
	local icon, iconsize = geticon(icon)
	data:extend {{
		type = "technology",
		name = name,
		icon = icon,
		icon_size = iconsize,
		max_level = max,
		unit = {
			count_formula = packs[1],
			time = packs[2],
			ingredients = getpacks(packs, 3)
		},
		effects = geteffect(effect),
		prerequisites = pre or {},
		order = order or ""
	}}
	return data.raw["technology"][name]
end
